@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.93 2019/09/27 10:26:32 claudio Exp $ */
+/*	$OpenBSD: config.c,v 1.95 2020/02/14 13:54:31 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -195,7 +195,6 @@ void
 merge_config(struct bgpd_config *xconf, struct bgpd_config *conf)
 {
 	struct listen_addr	*nla, *ola, *next;
-	struct network		*n;
 	struct peer		*p, *np, *nextp;
 
 	/*
@@ -250,10 +249,7 @@ merge_config(struct bgpd_config *xconf, struct bgpd_config *conf)
 
 	/* switch the network statements, but first remove the old ones */
 	free_networks(&xconf->networks);
-	while ((n = TAILQ_FIRST(&conf->networks)) != NULL) {
-		TAILQ_REMOVE(&conf->networks, n, entry);
-		TAILQ_INSERT_TAIL(&xconf->networks, n, entry);
-	}
+	TAILQ_CONCAT(&xconf->networks, &conf->networks, entry);
 
 	/* switch the l3vpn configs, first remove the old ones */
 	free_l3vpns(&xconf->l3vpns);
@@ -508,22 +504,6 @@ prepare_listeners(struct bgpd_config *conf)
 	return (r);
 }
 
-void
-copy_filterset(struct filter_set_head *source, struct filter_set_head *dest)
-{
-	struct filter_set	*s, *t;
-
-	if (source == NULL)
-		return;
-
-	TAILQ_FOREACH(s, source, entry) {
-		if ((t = malloc(sizeof(struct filter_set))) == NULL)
-			fatal(NULL);
-		memcpy(t, s, sizeof(struct filter_set));
-		TAILQ_INSERT_TAIL(dest, t, entry);
-	}
-}
-
 void
 expand_networks(struct bgpd_config *c)
 {
@@ -546,8 +526,7 @@ expand_networks(struct bgpd_config *c)
 				memcpy(&m->net.prefix, &psi->p.addr,
 				    sizeof(m->net.prefix));
 				m->net.prefixlen = psi->p.len;
-				TAILQ_INIT(&m->net.attrset);
-				copy_filterset(&n->net.attrset,
+				filterset_copy(&n->net.attrset,
 				    &m->net.attrset);
 				TAILQ_INSERT_TAIL(nw, m, entry);
 			}
